@@ -111,7 +111,8 @@ static bool runtime_sym_gvs(const char *f_lib, const char *f_name, MT &&M,
         llvmgv = new GlobalVariable(*M, T_pvoidfunc, false,
                                     GlobalVariable::ExternalLinkage, NULL, name);
         llvmgv = global_proto(llvmgv);
-        void *addr = jl_dlsym_e(libsym, f_name);
+        void *addr;
+        jl_dlsym(libsym, f_name, &addr, 0);
         (*symMap)[f_name] = std::make_pair(llvmgv, addr);
         if (symaddr)
             *symaddr = addr;
@@ -206,11 +207,7 @@ static Value *runtime_sym_lookup(
 // all the arguments without writing assembly directly.
 // This doesn't matter too much in reality since a single function is usually
 // not called with multiple signatures.
-#if JL_LLVM_VERSION >= 50000
 static DenseMap<AttributeList,
-#else
-static DenseMap<AttributeSet,
-#endif
                 std::map<std::tuple<GlobalVariable*,FunctionType*,
                                     CallingConv::ID>,GlobalVariable*>> allPltMap;
 
@@ -218,11 +215,7 @@ static DenseMap<AttributeSet,
 // when being called the first time.
 static GlobalVariable *emit_plt_thunk(
         Module *M, FunctionType *functype,
-#if JL_LLVM_VERSION >= 50000
         const AttributeList &attrs,
-#else
-        const AttributeSet &attrs,
-#endif
         CallingConv::ID cc, const char *f_lib, const char *f_name,
         GlobalVariable *libptrgv, GlobalVariable *llvmgv,
         void *symaddr, bool runtime_lib)
@@ -262,11 +255,7 @@ static GlobalVariable *emit_plt_thunk(
     // NoReturn function can trigger LLVM verifier error when declared as
     // MustTail since other passes might replace the `ret` with
     // `unreachable` (LLVM should probably accept `unreachable`).
-#if JL_LLVM_VERSION >= 50000
     if (attrs.hasAttribute(AttributeList::FunctionIndex,
-#else
-    if (attrs.hasAttribute(AttributeSet::FunctionIndex,
-#endif
                            Attribute::NoReturn)) {
         irbuilder.CreateUnreachable();
     }
@@ -299,11 +288,7 @@ static GlobalVariable *emit_plt_thunk(
 static Value *emit_plt(
         jl_codectx_t &ctx,
         FunctionType *functype,
-#if JL_LLVM_VERSION >= 50000
        const AttributeList &attrs,
-#else
-       const AttributeSet &attrs,
-#endif
        CallingConv::ID cc, const char *f_lib, const char *f_name)
 {
     assert(imaging_mode);
@@ -453,11 +438,7 @@ static Value *llvm_type_rewrite(
     // sizes.
     Value *from;
     Value *to;
-#if JL_LLVM_VERSION >= 40000
     const DataLayout &DL = jl_data_layout;
-#else
-    const DataLayout &DL = jl_ExecutionEngine->getDataLayout();
-#endif
     if (DL.getTypeAllocSize(target_type) >= DL.getTypeAllocSize(from_type)) {
         to = emit_static_alloca(ctx, target_type);
         from = emit_bitcast(ctx, to, from_type->getPointerTo());
@@ -675,9 +656,9 @@ static void interpret_symbol_arg(jl_codectx_t &ctx, native_sym_arg_t &out, jl_va
         jl_cgval_t arg1 = emit_expr(ctx, arg);
         jl_value_t *ptr_ty = arg1.typ;
         if (!jl_is_cpointer_type(ptr_ty)) {
-           const char *errmsg = !strcmp(fname, "ccall") ?
-               "ccall: first argument not a pointer or valid constant expression" :
-               "cglobal: first argument not a pointer or valid constant expression";
+            const char *errmsg = !strcmp(fname, "ccall") ?
+                "ccall: first argument not a pointer or valid constant expression" :
+                "cglobal: first argument not a pointer or valid constant expression";
             emit_cpointercheck(ctx, arg1, errmsg);
         }
         arg1 = update_julia_type(ctx, arg1, (jl_value_t*)jl_voidpointer_type);
@@ -796,8 +777,8 @@ static jl_cgval_t emit_cglobal(jl_codectx_t &ctx, jl_value_t **args, size_t narg
             res = ctx.builder.CreatePtrToInt(res, lrt);
         }
         else {
-            void *symaddr = jl_dlsym_e(jl_get_library(sym.f_lib), sym.f_name);
-            if (symaddr == NULL) {
+            void *symaddr;
+            if (!jl_dlsym(jl_get_library(sym.f_lib), sym.f_name, &symaddr, 0)) {
                 std::stringstream msg;
                 msg << "cglobal: could not find symbol ";
                 msg << sym.f_name;
@@ -1018,7 +999,10 @@ static jl_cgval_t emit_llvmcall(jl_codectx_t &ctx, jl_value_t **args, size_t nar
         jl_value_t *tti = jl_svecref(tt,i);
         bool toboxed;
         Type *t = julia_type_to_llvm(tti, &toboxed);
-        argtypes.push_back(t);
+        if (toboxed)
+            argtypes.push_back(T_prjlvalue);
+        else
+            argtypes.push_back(t);
         if (4 + i > nargs) {
             jl_error("Missing arguments to llvmcall!");
         }
@@ -1033,6 +1017,8 @@ static jl_cgval_t emit_llvmcall(jl_codectx_t &ctx, jl_value_t **args, size_t nar
     Function *f;
     bool retboxed;
     Type *rettype = julia_type_to_llvm(rtt, &retboxed);
+    if (retboxed)
+        rettype = T_prjlvalue;
     if (isString) {
         // Make sure to find a unique name
         std::string ir_name;
@@ -1083,15 +1069,10 @@ static jl_cgval_t emit_llvmcall(jl_codectx_t &ctx, jl_value_t **args, size_t nar
         << jl_string_data(ir) << "\n}";
         SMDiagnostic Err = SMDiagnostic();
         std::string ir_string = ir_stream.str();
-#if JL_LLVM_VERSION >= 60000
         // Do not enable update debug info since it runs the verifier on the whole module
         // and will error on the function we are currently emitting.
         bool failed = parseAssemblyInto(llvm::MemoryBufferRef(ir_string, "llvmcall"),
                                         *jl_Module, Err, nullptr, /* UpdateDebugInfo */ false);
-#else
-        bool failed = parseAssemblyInto(llvm::MemoryBufferRef(ir_string, "llvmcall"),
-                                        *jl_Module, Err);
-#endif
         if (failed) {
             std::string message = "Failed to parse LLVM Assembly: \n";
             llvm::raw_string_ostream stream(message);
@@ -1160,11 +1141,7 @@ static jl_cgval_t emit_llvmcall(jl_codectx_t &ctx, jl_value_t **args, size_t nar
 static Value *box_ccall_result(jl_codectx_t &ctx, Value *result, Value *runtime_dt, jl_value_t *rt)
 {
     // XXX: need to handle parameterized zero-byte types (singleton)
-#if JL_LLVM_VERSION >= 40000
     const DataLayout &DL = jl_data_layout;
-#else
-    const DataLayout &DL = jl_ExecutionEngine->getDataLayout();
-#endif
     unsigned nb = DL.getTypeStoreSize(result->getType());
     MDNode *tbaa = jl_is_mutable(rt) ? tbaa_mutab : tbaa_immut;
     Value *strct = emit_allocobj(ctx, nb, runtime_dt);
@@ -1192,11 +1169,7 @@ public:
     std::vector<bool> fargt_isboxed; // vector of whether the llvm output type is a Julia-box for each argument (vararg is the last item, if applicable)
     Type *fargt_vasig = NULL; // ABI coercion type for vararg list
     std::vector<bool> byRefList; // vector of "byref" parameters (vararg is the last item, if applicable)
-#if JL_LLVM_VERSION >= 50000
     AttributeList attributes; // vector of function call site attributes (vararg is the last item, if applicable)
-#else
-    AttributeSet attributes; // vector of function call site attributes (vararg is the last item, if applicable)
-#endif
     Type *lrt; // input parameter of the llvm return type (from julia_struct_to_llvm)
     bool retboxed; // input parameter indicating whether lrt is jl_value_t*
     Type *prt; // out parameter of the llvm return type for the function signature
@@ -1239,11 +1212,7 @@ std::string generate_func_sig(const char *fname)
     size_t nargt = jl_svec_len(at);
     assert(rt && !jl_is_abstract_ref_type(rt));
 
-#if JL_LLVM_VERSION >= 50000
     std::vector<AttrBuilder> paramattrs;
-#else
-    std::vector<AttributeSet> paramattrs;
-#endif
     std::unique_ptr<AbiLayout> abi;
     if (llvmcall)
         abi.reset(new ABI_LLVMLayout());
@@ -1266,11 +1235,7 @@ std::string generate_func_sig(const char *fname)
             retattrs.addAttribute(Attribute::StructRet);
 #endif
             retattrs.addAttribute(Attribute::NoAlias);
-#if JL_LLVM_VERSION >= 50000
             paramattrs.push_back(std::move(retattrs));
-#else
-            paramattrs.push_back(AttributeSet::get(jl_LLVMContext, 1, retattrs));
-#endif
             fargt_sig.push_back(PointerType::get(lrt, 0));
             sret = 1;
             prt = lrt;
@@ -1348,11 +1313,7 @@ std::string generate_func_sig(const char *fname)
 
         do { // for each arg for which this type applies, add the appropriate LLVM parameter attributes
             if (i < nargs) { // if vararg, the last declared arg type may not have a corresponding arg value
-#if JL_LLVM_VERSION >= 50000
                 AttributeSet params = AttributeSet::get(jl_LLVMContext, ab);
-#else
-                AttributeSet params = AttributeSet::get(jl_LLVMContext, i + sret + 1, ab);
-#endif
                 paramattrs.push_back(params);
             }
             i++;
@@ -1361,22 +1322,13 @@ std::string generate_func_sig(const char *fname)
 
     for (i = 0; i < nargs + sret; ++i) {
         const auto &as = paramattrs.at(i);
-#if JL_LLVM_VERSION >= 50000
         if (!as.hasAttributes())
             continue;
-#else
-        if (as.isEmpty())
-            continue;
-#endif
         attributes = attributes.addAttributes(jl_LLVMContext, i + 1, as);
     }
     if (rt == jl_bottom_type) {
         attributes = attributes.addAttribute(jl_LLVMContext,
-#if JL_LLVM_VERSION >= 50000
                                              AttributeList::FunctionIndex,
-#else
-                                             AttributeSet::FunctionIndex,
-#endif
                                              Attribute::NoReturn);
     }
     return "";
@@ -1544,14 +1496,19 @@ static jl_cgval_t emit_ccall(jl_codectx_t &ctx, jl_value_t **args, size_t nargs)
     };
 #define is_libjulia_func(name) _is_libjulia_func((uintptr_t)&(name), #name)
 
+    static jl_ptls_t (*ptls_getter)(void) = [] {
+    // directly accessing the address of an ifunc can cause compile-time linker issues
+    // on some configurations (e.g. AArch64 + -Bsymbolic-functions), so we guard the
+    // `&jl_get_ptls_states` within this `#ifdef` guard, and use a more roundabout
+    // method involving `jl_dlsym()` on Linux platforms instead.
 #ifdef _OS_LINUX_
-    // directly accessing the address of an ifunc can cause linker issue on
-    // some configurations (e.g. AArch64 + -Bsymbolic-functions).
-    static const auto ptls_getter = jl_dlsym_e(jl_dlopen(nullptr, 0),
-                                               "jl_get_ptls_states");
+        jl_ptls_t (*p)(void);
+        jl_dlsym(jl_dlopen(nullptr, 0), "jl_get_ptls_states", (void **)&p, 0);
+        return p;
 #else
-    static const auto ptls_getter = &jl_get_ptls_states;
+        return &jl_get_ptls_states;
 #endif
+    }();
 
     // emit arguments
     jl_cgval_t *argv = (jl_cgval_t*)alloca(sizeof(jl_cgval_t) * nccallargs);
@@ -2021,8 +1978,8 @@ jl_cgval_t function_sig_t::emit_a_ccall(
                 llvmf = emit_plt(ctx, functype, attributes, cc, symarg.f_lib, symarg.f_name);
         }
         else {
-            void *symaddr = jl_dlsym_e(jl_get_library(symarg.f_lib), symarg.f_name);
-            if (symaddr == NULL) {
+            void *symaddr;
+            if (!jl_dlsym(jl_get_library(symarg.f_lib), symarg.f_name, &symaddr, 0)) {
                 std::stringstream msg;
                 msg << "ccall: could not find function ";
                 msg << symarg.f_name;
@@ -2105,11 +2062,7 @@ jl_cgval_t function_sig_t::emit_a_ccall(
                 MDNode *tbaa = jl_is_mutable(rt) ? tbaa_mutab : tbaa_immut;
                 int boxalign = jl_datatype_align(rt);
                 // copy the data from the return value to the new struct
-#if JL_LLVM_VERSION >= 40000
                 const DataLayout &DL = jl_data_layout;
-#else
-                const DataLayout &DL = jl_ExecutionEngine->getDataLayout();
-#endif
                 auto resultTy = result->getType();
                 if (DL.getTypeStoreSize(resultTy) > rtsz) {
                     // ARM and AArch64 can use a LLVM type larger than the julia type.
