@@ -57,10 +57,15 @@ end
     @test median!([1 2; 3 4]) == 2.5
 
     @test invoke(median, Tuple{AbstractVector}, 1:10) == median(1:10) == 5.5
+
+    @test @inferred(median(Float16[1, 2, NaN])) === Float16(NaN)
+    @test @inferred(median(Float16[1, 2, 3]))   === Float16(2)
+    @test @inferred(median(Float32[1, 2, NaN])) === NaN32
+    @test @inferred(median(Float32[1, 2, 3]))   === 2.0f0
 end
 
 @testset "mean" begin
-    @test_throws ArgumentError mean(())
+    @test_throws MethodError mean(())
     @test mean((1,2,3)) === 2.
     @test mean([0]) === 0.
     @test mean([1.]) === 1.
@@ -86,6 +91,21 @@ end
     @test ismissing(mean([missing, NaN]))
     @test isequal(mean([missing 1.0; 2.0 3.0], dims=1), [missing 2.0])
     @test mean(skipmissing([1, missing, 2])) === 1.5
+    @test isequal(mean(Complex{Float64}[]), NaN+NaN*im)
+    @test mean(Complex{Float64}[]) isa Complex{Float64}
+    @test isequal(mean(skipmissing(Complex{Float64}[])), NaN+NaN*im)
+    @test mean(skipmissing(Complex{Float64}[])) isa Complex{Float64}
+    @test isequal(mean(abs, Complex{Float64}[]), NaN)
+    @test mean(abs, Complex{Float64}[]) isa Float64
+    @test isequal(mean(abs, skipmissing(Complex{Float64}[])), NaN)
+    @test mean(abs, skipmissing(Complex{Float64}[])) isa Float64
+    @test isequal(mean(Int[]), NaN)
+    @test mean(Int[]) isa Float64
+    @test isequal(mean(skipmissing(Int[])), NaN)
+    @test mean(skipmissing(Int[])) isa Float64
+    @test_throws MethodError mean([])
+    @test_throws MethodError mean(skipmissing([]))
+    @test_throws ArgumentError mean((1 for i in 2:1))
 
     # Check that small types are accumulated using wider type
     for T in (Int8, UInt8)
@@ -104,15 +124,17 @@ end
             @test f(2:0.1:n) ≈ f([2:0.1:n;])
         end
     end
+    @test mean(2:1) === NaN
+    @test mean(big(2):1) isa BigFloat
 end
 
 @testset "var & std" begin
     # edge case: empty vector
     # iterable; this has to throw for type stability
-    @test_throws ArgumentError var(())
-    @test_throws ArgumentError var((); corrected=false)
-    @test_throws ArgumentError var((); mean=2)
-    @test_throws ArgumentError var((); mean=2, corrected=false)
+    @test_throws MethodError var(())
+    @test_throws MethodError var((); corrected=false)
+    @test_throws MethodError var((); mean=2)
+    @test_throws MethodError var((); mean=2, corrected=false)
     # reduction
     @test isnan(var(Int[]))
     @test isnan(var(Int[]; corrected=false))
@@ -245,6 +267,18 @@ end
         @test ismissing(f([missing, NaN], missing))
         @test f(skipmissing([1, missing, 2]), 0) === f([1, 2], 0)
     end
+
+    @test isequal(var(Complex{Float64}[]), NaN)
+    @test var(Complex{Float64}[]) isa Float64
+    @test isequal(var(skipmissing(Complex{Float64}[])), NaN)
+    @test var(skipmissing(Complex{Float64}[])) isa Float64
+    @test_throws MethodError var([])
+    @test_throws MethodError var(skipmissing([]))
+    @test_throws MethodError var((1 for i in 2:1))
+    @test isequal(var(Int[]), NaN)
+    @test var(Int[]) isa Float64
+    @test isequal(var(skipmissing(Int[])), NaN)
+    @test var(skipmissing(Int[])) isa Float64
 end
 
 function safe_cov(x, y, zm::Bool, cr::Bool)
